##!perl -w
#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2010-2011 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################
#trtex.exe launcher is located in bin and execute the perl sript trtex.pl located in the lib/scripts
#it is used by ECDT to execute a targeted instrumented application 
################################################################################
#$Id: trtex.pl,v 1.1.2.8 2011/04/07 14:39:37 nmezy Exp $
################################################################################

# useful for stdout/stderr direct viewing inside 'console' of studio window
# (better: we have stdout/stderr lines in right order and not stderr before stdout
use FileHandle ;
STDOUT->autoflush(1);
STDERR->autoflush(1);

die "Environment variable TESTRTDIR not found. Exiting.\n" if(!defined($ENV{'TESTRTDIR'})); # NO TRANSLATION
$main::SCRIPTDIR="$ENV{'TESTRTDIR'}/lib/scripts/";
require $SCRIPTDIR . "Fcts.pl";
require $SCRIPTDIR . "Tools.pl";
require $SCRIPTDIR . "Split.pl";

sub trtReadArgs
{

	## globals ############
	$main::TestRTCC=1;
	#######################
	$OUTDIR='.';
	$INIFILE='tp.ini';
	$STAGE ='l';
	$LOGFILENAME='trtld.log';
	require $SCRIPTDIR ."BatchCCDefaults.pl";
	$TARGETDIR=$ENV{ATLTGT};
	#
	# trtex -config <envNnode.pl> -p <TESTPROGRAM> <ARGS> 
	#
	#  INTDIR is set from -o option if not yet specified in the envNode.pl
	$_=shift(@ARGV);
	if(/^-config/){
		$nodeSettings=shift(@ARGV);
		$nodeSettings=~s/\"//g;
		die "$nodeSettings not found. Unable to Launch command.\n" unless (-f $nodeSettings);
		require $nodeSettings;
	} else {
	    print &PerlTranslate("trtex.pl", "trtex usage -config <envNode.pl> [-p <TESTPROGRAM>] [<ARGS>]\n", "usage");
	    exit(1);
	}
	if($ARGV[0] eq '-p'){
		shift(@ARGV);
		$TESTPROGRAM=shift(@ARGV);
	}
	$TESTPROGRAM=~s/\"//g;
	$ARGUMENTS=join(' ',@ARGV).$ARGUMENTS;
	@ARGV=();
	
}


sub trtExec
{
my @Splitted=();

# Change Dir to the configuration folder
chdir($CONFIGURATION);

my $CurrentDir=GetCurrentDir();
	print &PerlTranslate("trtex.pl", "Executing %1 %2...\n", "%1 is the program to be executed and %2 contains the arguments given to it", $TESTPROGRAM, $ARGUMENTS);
	print &PerlTranslate("trtex.pl", "from %1\n", "%1 is the current directory", $CurrentDir);
	my $status = AtlExec ($TESTPROGRAM, $ATL_OUT_FILE, $ARGUMENTS);
	EndStep($status);
        $status=0; #status is shown but not used 

	if ( $ATCMODE || $ATTMODE || $ATPMODE || $ATQMODE || $ATKMODE )
	{
	       # when socket uploader is used SplitTraces check created files and make the log 
	       my $exeInstr=0;
		  if($ATL_MULTI_PROCESS){
			 my ($dir,$name,$ext) = SplitFileName($ATL_OUT_FILE);
			 opendir(DIR,"$dir") || die &PerlTranslate("trtex.pl", "Error: unable to open directory %1.\n","%1 directory name",$dir);
			 while($f=readdir(DIR)) {
				SplitTraces ($dir.'/'.$f) if($f=~ /$name\..*\.$ext/);
			 }
			 closedir(DIR);
		  } else {
			SplitTraces ($ATL_OUT_FILE); # LOG are made inside SplitTraces
		  }
		  #---------------------------------------------------------------------
		  ## Coverage check ...
		  if ( $ATCMODE )
		  {
		      if ( ! $used[$tio] )
		      {
			  print &PerlTranslate("trtex.pl", "No Coverage results found.\n");
			  $exeInstr++;
		      }
		      #atc_postpro($ATC_TIO_NAME,""); # no CIO in this node
		  } 	  

		  #---------------------------------------------------------------------
		  ## Manage TDF Traces ...
		  if ( $ATTMODE )
		  { # No attsplit nor attloop on the fly
		      if ( $used[$tdf] )
		      {
			      # print $TDFFILE, "size=",(-s $TDFFILE),"\n";
			      if((-s $TDFFILE) > ($ATT_SPLIT_SIZE*1000)) {
				  # get the TSF file in the LOG to be used by the PostAttManage
				  readLogFile() unless(@TSFFILES);

				  #then split the big file
				  PostAttManage ($TDFFILE);
				  push(@Splitted,$TDFFILE);
			      }
		      }
		      else
		      {
			  print &PerlTranslate("trtex.pl", "No Tracing results found.\n");
			  $exeInstr++;
		      }
		  } #ATTMODE==1

		  #---------------------------------------------------------------------
		  ## Memory Profiling check ...
		  if ( $ATPMODE )
		  {
		      if ( ! $used[$tpf] )
		      {
			  print &PerlTranslate("trtex.pl", "No Mempro results found.\n");
		      }
		  } #ATPMODE==1

		  #---------------------------------------------------------------------------
		  ## Performance Profiling check ...
		  if ( $ATQMODE )
		  {
		      if ( ! $used[$tqf] )
		      {
			  print &PerlTranslate("trtex.pl", "No Perfpro results found.\n");
			  $exeInstr++;
		      }
		  } #ATQMODE==1

		  #---------------------------------------------------------------------------
		if($exeInstr == 3) {
		  print "No results. Clean the project to instrument the application.\n";
		  exit(1);
		}
	    } #ATCMODE==1 || ATTMODE==1 || ATPMODE==1 || ATQMODE==1 || $ATKMODE==1

	#clean the logfile.
	cleanLogFile(@Splitted);
return $status
}

sub readLogFile
{
my $d=".";
  if(open IN, $LOGFILENAME) {
      while(<IN>){
	chomp;
	if(/^DIR\s+(.*)$/) {$d=$1;}
	if(/^TSF\s+(.*)$/) {my $f=$1; $f=$d.'/'.$f unless(-f $f); push(@TSFFILES,$f);}
	if(/^FDC\s+(.*)$/) {my $f=$1; $f=$d.'/'.$f unless(-f $f); push(@FDCFILES,$f);}
      }
      close IN;
  }
}

sub cleanLogFile
{
my @rmFiles=@_;
my $currentDir="";
 if(open IN, $LOGFILENAME) {
    my $tsf=( $ATTMODE || $ATPMODE || $ATQMODE || $ATKMODE );
    my %seen = (); 
    foreach(@rmFiles){$seen{$_}++;}
    open OU, ">$LOGFILENAME.tmp" or die &PerlTranslate("trtex.pl", "cannot write the temp log file.\n");
    while(<IN>){
	chomp;
	if(/(\w{3})\s+(.+)$/){
	   my $tk=$1;
	   if(($currentDir=="") && ($tk eq 'DIR')) {
			$currentDir=$2;
	   }
	   my $pt=absolutePath($2,$currentDir);
	   unless( $seen{$pt}++){
		if(($tk eq 'DUM') && !($KEEP)) {
			# unlink absolute and relative files
			if (-f $pt) {
				unlink($pt);
#			} else {
#				unlink($currentDir."/".$2);
			}
		}
		
		else {print OU "$tk $2\n" ;}
	   }
	} else {
	    print "unknown line in log file $_\n";
	}
   }
   close IN;
   close OU;
   rename($LOGFILENAME.'.tmp',$LOGFILENAME);
 } else {
	&PerlTranslate("trtex.pl", "No result log found.\n");
 }
}
################################################################################
# main
################################################################################
unless($main::TestRTCC){

  trtReadArgs();
  die "TDP not found.\n" unless(-f $TARGETDIR . "/cmd/atl_exec.pl");

  require $TARGETDIR . "/cmd/atl_func.pl";
  require $TARGETDIR . "/cmd/atl_exec.pl";
  EnterInScript();
  trtExec();
  exit(0);
}
1;

