/*
  Licensed Materials - Property of IBM
  Restricted Material of IBM
  IBM Rational Test RealTime
  (c) Copyright IBM Corp. 2003-2008 All Rights Reserved.

  US Governement Users Restricted Rights - Use, duplication or
  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*/
/* AtsMain.c for windows V1.4 richard.france */
/* This AtsMain.c[pp] is a thread launcher for system testing
	It should be located in the targets/<target>/lib directory
	It is used when Virtual testers are generated as threads and this file exist in the TDP
	The execution command line format is -thread=rioFileName,instanceName,occurence -thread=.....
*/
#include <windows.h>
#include <winbase.h>


/*! Expected argument type in start routine of thread mode name of rio file
 * to generate list of filters set. */
#include "atsapi.h"

#ifdef ATTOLSTUDIO_VERBOSE
#define trace printf
#endif
#ifndef trace
#define trace /##/
#endif

#ifdef __cplusplus
#define EXTERN extern "C"
#else
#define EXTERN extern
#endif
EXTERN int atl_strncmp(char*, char*, int);

typedef struct {
	ATL_T_ARG	ArgThread;
	HANDLE 		HndThread;
} ATL_T_Thread, *ATL_PT_Thread;

DWORD ThreadId;
DWORD resultThread;
/* modification for visual */
#define ATLStart(x) ATL_Start
#define THREADSTABLELEN ATLStart(x)##_TableLen
#define THREADSTABLE ATLStart(x)##_Table

#ifdef THREADSTABLELEN
extern int THREADSTABLELEN;
extern char * THREADSTABLE [];
#endif
extern ATL_T_THREAD_RETURN  ATL_Start(ATL_PT_ARG arg);

#ifndef ATL_THREAD_STACK_SIZE
#define ATL_THREAD_STACK_SIZE 0
#endif
#ifndef ATL_THREAD_PRIORITY
#define ATL_THREAD_PRIORITY	THREAD_PRIORITY_ABOVE_NORMAL
#endif

#ifdef RTRT_FAST_RDV
/* to use this local RDV */
/* Use Build setting : compil : macro definition = RTRT_FAST_RDV , and */
/* Use System testing setting: Advenced options:Run without deployment: YES */
/* local RDV allows to use RDV without deployment when RDV are between threads of the same process */
/* rdv declaration */
#define EVENT_AVAILABLE 1
/* if Event not available on your OS set EVENT_AVAILABLE to 0 */
#if EVENT_AVAILABLE
#define RDV_EVENT HANDLE
#define LockThread(e,t) WaitForSingleObject(e,t)
#define stopThread(e) ResetEvent(e)
#define startThread(e) SetEvent(e)
#else
#define RDV_SLEEP(t) sleep(t)
#define RDV_EVENT char
#define RDV_DELAY 10
#define LockThread(e,t) {int tt=t/RDV_DELAY; while(!e && (--tt)){RDV_SLEEP(RDV_DELAY);}
#define stopThread(e) e=0
#define startThread(e) e=1
#endif
typedef struct {
	char * RdvName;
	int TotalMembers;
	int ActualMembers;
	HANDLE Hevent;
} ATL_T_RDV;

#include "rdvtable.h"
/* content format for the rdvtable.h
ATL_T_RDV rdvList[]={
{"rdvName",members,0,NULL},
{"rdvName",members},
};
#define ATL_RDVTIMEOUT 30000 // for 30 seconds
*/
/* ATL_RDVTIMEOUT is defined into the rdvtable.h if RENDEZVOUS is found in the script */
#ifdef ATL_RDVTIMEOUT
#define NB_RDV sizeof(rdvList)/sizeof(ATL_T_RDV)
EXTERN int usr_rdv (char *token_name);
#define ATL_RDVMUTEX "ATL_RDVMUTEX"
#define ATL_RDVMUTEXTIMEOUT 3000
HANDLE atl_rdvMutex;
#endif
#endif  /* RTRT_FAST_RDV */

#include <stdio.h>
int main(int argc, char **argv)
{
 int threadNo=0;
 int NB_THREAD=0;
 ATL_PT_Thread pThread,pThread1;
 char *c,*p;
 if(argc >1)
   {
 	  pThread1=malloc(argc*sizeof(ATL_T_Thread));
 	  pThread=pThread1;
	  for (threadNo=1 ; threadNo < argc ;threadNo++)
	   {
		if ((atl_strncmp(argv[threadNo], "-THREAD=", 8) == 0) || (atl_strncmp(argv[threadNo], "-thread=", 8) == 0))
		 {
			  p=&argv[threadNo][8];
			  pThread->ArgThread.atl_riofilename=p;
			  pThread->ArgThread.atl_filters="";
			  for(; *p && *p!=',';p++);
			  *p++=0;
			  pThread->ArgThread.atl_instance=p;
			  for(; *p && *p!=',';p++);
			  *p++=0;
			  if(*p) pThread->ArgThread.atl_occid=atoi(p);
		 	  NB_THREAD++;
		 	  pThread++;
		 }
	   }
	   trace("found %d instances on cmd line\n",NB_THREAD);
   }
#ifdef THREADSTABLELEN
   else
   {
	 NB_THREAD=THREADSTABLELEN;
	 pThread1=malloc(NB_THREAD*sizeof(ATL_T_Thread));
	 pThread=pThread1;
	 threadNo=0;
	 while(threadNo<NB_THREAD)
	 {
		   c=THREADSTABLE[threadNo];
		   p=malloc(strlen(c)+4+1);
		   pThread->ArgThread.atl_instance=c;
		   pThread->ArgThread.atl_riofilename=p;
		   while(*c){*p++=*c++;}
		   c=".rio";
		   while(*c){*p++=*c++;}
		   *p=0;
		   pThread->ArgThread.atl_filters="";
		   pThread->ArgThread.atl_occid=0;
		trace("Instance %s => %s \n",pThread->ArgThread.atl_instance, pThread->ArgThread.atl_riofilename);
		   pThread++;
		   threadNo++;
	 }
   }
#endif
/*--------------RDV creation-----------------*/
#ifdef NB_RDV
	atl_rdvMutex=CreateMutex(NULL,FALSE,ATL_RDVMUTEX);
#if EVENT_AVAILABLE
	while(threadNo<NB_RDV)
	{
		rdvList[threadNo].Hevent=CreateEvent(NULL,TRUE,FALSE,rdvList[threadNo].RdvName);
		threadNo++;
	}
#endif /* EVENT_AVAILABLE */

#endif
/*-------------------------------------------*/
	/* threads creation */
	threadNo=0;
	pThread=pThread1;
	while(threadNo<NB_THREAD)
	{
		pThread->HndThread = CreateThread(
		   NULL,  /* pointer to security attributes */
		   ATL_THREAD_STACK_SIZE,     /* initial thread stack size */
		   (LPTHREAD_START_ROUTINE) ATL_Start,/* pointer to thread function */
		   (void *)&pThread->ArgThread,     /* argument for new thread */
		   0,     /* creation flags */
		   &ThreadId);/* pointer to receive thread ID);*/
		trace("Instance %s id:%d\n",pThread->ArgThread.atl_instance, pThread->HndThread);
		SetThreadPriority(pThread->HndThread,ATL_THREAD_PRIORITY);
		threadNo++;
 	    pThread++;

	    Sleep(50);
	}
	/* wait threads end */
	threadNo=0;
	pThread=pThread1;
	while(threadNo<NB_THREAD)
	{	 		/*_cwait(&resultThread, HndThread[threadNo], NULL); */
		GetExitCodeThread(pThread->HndThread, &resultThread);
		if(resultThread == STILL_ACTIVE)
		{
			Sleep(1000);
		}
		else
		{
			printf("instance %s (No %d) exit code: %8X\n", pThread->ArgThread.atl_instance, pThread->HndThread,resultThread);
 	        pThread++;
			threadNo++;
		}
	}
	return 0;
}

#ifdef RTRT_FAST_RDV
int priv_rdv (char *token_name)
{
#ifdef ATL_RDVTIMEOUT
ATL_T_RDV *plist;
int x=0;
int res;

	while(x<NB_RDV)
	{
		plist=&rdvList[x++];
		if(strcmp(plist->RdvName,token_name)==0) /* identical */
		{
			switch(res=WaitForSingleObject(atl_rdvMutex,ATL_RDVMUTEXTIMEOUT)) /* lock mutex */
			{
			case WAIT_TIMEOUT :
			case WAIT_FAILED:
				{
					printf("MUTEX RDV access problem\n");
					return (-1);
				}
			}
			trace("%s RDV (%d)\n",token_name,plist->ActualMembers+1);
			if(++(plist->ActualMembers) >= plist->TotalMembers)
			{ /* everybody at RENDEZVOUS: release */
				plist->ActualMembers=0;
				ReleaseMutex(atl_rdvMutex);
				startThread(plist->Hevent); /* unlock other threads */
				return (0);
			}
			if(plist->ActualMembers==1)
			{
				stopThread(plist->Hevent); /* set to stop for threads */
			}
			ReleaseMutex(atl_rdvMutex);
			LockThread(plist->Hevent,ATL_RDVTIMEOUT); /* lock thread : wait other members  */
			if(plist->ActualMembers ==0)
				return (0);
			if(plist->ActualMembers >0)
			{
					plist->ActualMembers=-plist->TotalMembers;
					printf("TIMEOUT_PROBLEM on RDV %s\n",plist->RdvName);
					startThread(plist->Hevent);
			}
			return (-1);
		}
	}
#endif
	printf("unknown RENDEZVOUS %s\n",token_name);
    return (-1);
}

#endif /* RTRT_FAST_RDV */

